; LCD test for C8051F967

$NOMOD51
$include (C8051F960.inc)      
Period_low		EQU 0x00		; 0 for 1 sek
Period_high		EQU 0x40		; 0x80 for 1 sek
ACTIVE_TIME		EQU 240		; max number of measurements
FILTER_ORDER	EQU 4			; running average digital filter setup	

 
	DSEG 	AT 0x20			; data segment in direct addressable space
dig1:		DS 1
dig2:		DS 1				; storage for digits
dig3:		DS 1
dig4:		DS 1
count:	DS 1				; active time counter
bcd:		DS 2				; buffer for BCD conversion

	ORG 	0x40
Buffer:	DS FILTER_ORDER*2		; circular buffer

	CSEG 	AT 0				; interrupt vectors
	ljmp 	main  			; reset

	ORG	0x83				; Port match vector
	mov	EIE2, #0			; disable port match int
	clr	IE.7				; disable interrupts
	reti

	ORG	0xBB				; code section right after the int. vectors
	USING    0                    ; specify register bank 

main:       
	anl   PCA0MD, #NOT(0x40)      ; disable the WDT
	mov	SP, #0x7F			; setup stack in indirect storage area

	acall	Ports_Setup
	acall	Clock_Setup
	acall	Debug_Trap			; debug trap !!!
	acall	RTC_Setup
	acall ADC_Setup

	 
wait: 					; MAIN LOOP
	mov	EIE2, #0x02			; enable Port mismatch interrupt
	setb	IE.7				; enable interrupts
	mov 	PMU0CF, #CLEAR		; erase wake-up events
	mov	PMU0CF, #(SLEEP OR PMATWK) ; set wakeup sources & sleep
	nop   				; device is now awake
  	nop
	nop
	nop
   	mov	RTC0ADR, #RTC0CF		; clear SmaRTClock alarm IF
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)	
	acall	System_ON			; turn on LCD		

loop:						; weight measure loop
   	mov	RTC0ADR, #RTC0CF		; clear SmaRTClock alarm IF
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)

	mov 	PMU0CF, #CLEAR		; erase wake-up events
	mov	PMU0CF, #(SLEEP OR RTCAWK) ; set wakeup sources & sleep
	nop   				; device is now awake
  	nop
	nop
	nop
   	mov	RTC0ADR, #RTC0CF		; clear SmaRTClock alarm IF
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)

	acall	Get_Weight			; get weight in R5:R4
	acall	Average_Weight		; R5:R4 = averaged weight
	acall	Display_Number		; display weight on LCD
	djnz	count, loop			; update active time counter

	acall	System_OFF			; blank LCD and sleep
	sjmp	wait

;-------------------PROCEDURES-------------------------------------------
System_ON:
	mov	R0, #Buffer			; init circular buffer values
	mov	R2, #FILTER_ORDER*2	; circ buffer size
so1:	mov	@R0, #0			; zero the buffer byte
	inc	R0				; update buffer pointer
	djnz	R2, so1			; loop back
	mov	R0, #Buffer
	mov	R3, #0			; R3:R2 = circ buffer sum
	mov	R2, #0
	
	mov	dig1, #0
	mov	dig2, #0
	mov	dig3, #0
	mov	dig4, #0
	acall	LCD_Setup
	mov	R5, #0
	mov	R4, #0	
	acall	Display_Number
	mov	count, #ACTIVE_TIME	; init awake time counter
	ret

System_OFF:
	clr	A
	mov	SFRPAGE,   #0x2		; switch to SFR page 0x2
	mov	LCD0D0, A			; blank LCD
	mov	LCD0D1, A
	mov	LCD0D2, A
	mov	LCD0D3, A
	mov	LCD0MSCN,  #0x0		; disable LCD driver
	; Contrast Control Mode 1	; charge pump is permanently off
	mov	LCD0MSCF, #0xFD		; bypass charge pump
	mov	LCD0PWR,  #0x01		; set contrast mode 1
	mov	LCD0VBMCN,#0x00		; disable VBAT monitor
	mov	SFRPAGE,  #0x0		; switch to SFR page 0x0
	ret

;-----------------------------------------------------------------------
Average_Weight:				; R5:R4=new_weight  R3:R2=circ_sum
	clr	C				; R0 points to the oldest value
	mov	A, R2				; ACC = LSB(circ sum)
	subb	A, @R0
	mov	R2, A
	inc	R0
	mov	A, R3				; ACC = MSB(circ sum)
	subb	A, @R0
	mov	R3, A	

	mov	A, R5
	mov	@R0, A			; incert new weight into
	dec	R0				; the circ buffer
	mov	A, R4
	mov	@R0, A
	inc	R0
	inc	R0
	anl	0, #0x40+(FILTER_ORDER*2 -1) ; take R0 modulo ...

	mov	A, R4				; update circ sum
	add	A, R2
	mov	R2, A
	mov	A, R5
	addc	A, R3
	mov	R3, A

	mov	R5, A				; R5:R4 = circ sum
	mov	A, R2
	mov	R4, A
	mov	R1, #FILTER_ORDER		; divide circ sum by FILTER_ORDER
l2:	clr	C
	mov	A, R5
	rrc	A
	mov	R5, A
	mov	A, R4
	rrc	A
	mov	R4, A
	djnz	R1, l2
	ret

;-----------------------------------------------------------------------
Get_Weight:
   	mov	IREF0CN, #0x3F		; set 20 uA current
	mov	REF0CN, #0x18           ; enable high-speed ref
  	mov  	ADC0CF, #0xFA		; start with PGA = 0.5
	mov	ADC0CN, #0xC0		; enable ADC

	mov	A, #30			; 10 usec delay		
	dec	A				; 1 cycle
	nop					; 1 cycle
	nop					; 1 cycle
	jnz	$-3				; 2 cycles

	orl	ADC0CN, #0x10		; start conversion
	nop
	nop	
	mov	A, ADC0CN			; poll BUSY flag
	anl	A, #0x20
	jz	$-5    	

	mov	R5, ADC0H			; get conversion result
	mov	R4, ADC0L
	mov	ADC0CN, #0x00		; disable ADC

	clr	C
	mov	A, R4
	subb	A, #0xD0			; compare R5:R4 with 2000 (0x7D0)	
	mov	A, R5
	subb	A, #0x07
	jc	gw1				; jump in R5:R4 < 2000
	mov	R5, #0
	mov	R4, #0
	sjmp	gw2

gw1:	mov  	ADC0CF, #0xFB		; switch PGA = 1.0
	mov	ADC0CN, #0xC0		; enable ADC
	mov	A, #30			; 10 usec delay		
	dec	A				; 1 cycle
	nop					; 1 cycle
	nop					; 1 cycle
	jnz	$-3				; 2 cycles

	orl	ADC0CN, #0x10		; start conversion
	nop
	nop	
	mov	A, ADC0CN			; poll BUSY flag
	anl	A, #0x20
	jz	$-5    	

	mov	R5, ADC0H			; get conversion result
	mov	R4, ADC0L
	mov	ADC0CN, #0x00		; disable ADC

	clr	C				; compute 4095 - ADC value
	mov	A, #0xFF
	subb	A, R4
	mov	R4, A
	mov	A, #0x0F
	subb	A, R5
	mov	R5, A				; R5:R4 = result

gw2:	mov	REF0CN, #0			; turn off reference
	mov	IREF0CN, #0			; disable IREF0
	ret

;-----------------------------------------------------------------------
ADC_Setup:
    	mov  ADC0AC, #0x81		; 12-bit mode, 4 conversions
	mov  ADC0MX, #0x06		; select P0.6 as ADC input
	ret

;-----------------------------------------------------------------------
Ports_Setup:
    	mov  	P0MDOUT,   #0x2E		; configure all ports for output
	mov	P0MDIN,    #0x3F
	mov	P0,	     #0xD1		; input on P0.0, P0.4
	mov  	P0SKIP,    #0xFF
  	mov	P0MAT,     #0xFF		; capture falling edge int
    	mov	P0MASK,    #0x10		;  at P0.4

    	mov  	P1MDOUT,   #0xF3		; input on P1.7
	mov	P1MDIN,    #0xF3		; analog operation on XTAL pins
	mov	P1,        #0x0C
	mov  	P1SKIP,    #0xFF

    	mov  	P2MDOUT,   #0x0F		; LCD operation on P2.4-P2.6
	mov	P2MDIN,    #0x0F
	mov	P2,        #0xF0    	
    	mov  	P2SKIP,    #0xFF

    	mov  	SFRPAGE,   #0x0F
    	mov  	P3MDOUT,   #0x00		; LCD operation on P3.0-P3.6
	mov	P3MDIN,    #0x00
	mov	P3,	     #0xFF
    	mov  	P4MDOUT,   #0x01
    	mov  	SFRPAGE,   #0x00
	mov  	XBR2,      #0x40		; enable XBar and weak pull-ups
	ret

;-----------------------------------------------------------------------
Clock_Setup:
	mov	RTC0ADR, #RTC0XCN		; disable Automatic Gain Control
	mov	RTC0DAT, #0x60		; and enable Bias Doubling
	mov	RTC0ADR, #RTC0XCF		; set 12 pF load capacitance
	mov	RTC0DAT, #0x84		; and enable atostep feature
	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)
	acall	Debug_Trap			; wait for XCO to start
	mov	RTC0ADR, #(0x40 OR RTC0XCN)
sc_loop1:
	mov 	A, RTC0DAT						
	jnb	ACC.4, sc_loop1		; verify that XOSC is stable
	mov	RTC0ADR, #(0x40 OR RTC0XCF)
sc_loop2:
	mov 	A, RTC0DAT						
	jnb	ACC.6, sc_loop2		; verify capacitance ready bit
	mov	RTC0ADR, #RTC0XCN		; enable Automatic Gain Control
	mov	RTC0DAT, #0xC0		; and disable Bias Doubling

;	mov	RTC0ADR, #RTC0XCN		; setup for LFO
;	mov	RTC0DAT, #LFOEN		; enable LFO
;	nop
;  	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock
;	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)  

	mov	PMU0MD,  #0			; enable POR supply monitor
	mov 	RSTSRC,  #0x06		; enable MCD and VDD monitor
	anl	OSCICN,  #NOT(0x80)	; disable internal precision oscillator
	mov	REG0CN,  #0			; disable precision oscillator bias
	mov	OSCXCN,  #0			; disable external oscillator
	mov	CLKSEL,  #0x03		; select SmaRTClock with 1:1 clock divider
	mov	A, CLKSEL			; wait for the divider setting to be applied
	jnb	ACC.7, $-2			
	ret

;-----------------------------------------------------------------------
Debug_Trap:
	mov	R2, #0xFF
	mov	A,  #0xFF
dt_loop:					; nested loops for approx. 10 sec delay
	nop
	djnz	ACC, dt_loop
	dec	R2
	mov	A, R2
	jnz	dt_loop	
	jnb	P0.0, $			; wait for slave debug trap is over
	ret

;-----------------------------------------------------------------------
RTC_Setup:	
	mov	CLKSEL, #0x04		; select LPO with 1:1 clock divider
	orl	FLSCL,  #BYPASS		; set the one-shot bypass bit
	mov	A, CLKSEL			; wait for the divider setting to be applied
	jnb	ACC.7, $-2

	mov 	RTC0ADR, #CAPTURE0	; clear RTC timer
	mov 	RTC0DAT, #0			; autoincrement address
	mov 	RTC0DAT, #0
	mov 	RTC0DAT, #0
	mov 	RTC0DAT, #0

	mov	RTC0ADR, #RTC0CN		; set timer value
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0SET) 
	mov 	RTC0ADR, #(0x40 OR RTC0CN)
ts_loop:
	mov 	A, RTC0DAT						
	jb	ACC.1, ts_loop		; verify that timer is set

   	mov	RTC0ADR, #RTC0CN		; enable SmaRTClock timer
	mov	RTC0DAT, #(RTC0EN OR HSMODE OR RTC0TR)  

	mov	R6, #Period_low
	mov	R7, #Period_high
ALARM_Setup:
   	mov	RTC0ADR, #RTC0CF		; disable alarm while
   	mov	RTC0DAT, #0 		; updating the regs      
	mov 	RTC0ADR, #ALARM0B0	; load alarm regs       
	mov   RTC0DAT, R6
	mov	RTC0DAT, R7
	mov	RTC0DAT, #0 
	mov	RTC0DAT, #0
   	mov	RTC0ADR, #RTC0CF		; enable SmaRTClock alarm
	mov	RTC0DAT, #(ALRM0EN OR AUTORST)  
	ret

;-----------------------------------------------------------------------
LCD_Setup:
	mov	SFRPAGE,   #0x2		; switch to SFR page 0x2
	mov	LCD0CN,    #0x06		; set 4-mux, 1/3 bias mode
	mov	LCD0MSCN,  #0x56		; enable LCD clock and bias, clear LCD data
	mov	LCD0CNTRST,#0x13		; set 2.72V contrast

	; Contrast Control Mode 1	; charge pump is permanently off
;	mov	LCD0MSCF, #0xFD		; bypass charge pump
;	mov	LCD0PWR,  #0x01		; set contrast mode 1
;	mov	LCD0VBMCN,#0x00		; disable VBAT monitor

	; Contrast Control Mode 2	; minimum contrast mode
;	mov	LCD0MSCF, #0xFD		; bypass charge pump
;	mov	LCD0PWR,  #0x09		; set contrast mode 2
;	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	; Contrast Control Mode 3	; constant contrast mode
	mov	LCD0MSCF, #0xFC		; do not bypass charge pump
	mov	LCD0PWR,  #0x09		; set contrast mode 3
	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	; Contrast Control Mode 4	; auto-bypass mode
;	mov	LCD0MSCF, #0xFC		; do not bypass charge pump
;	mov	LCD0PWR,  #0x01		; set contrast mode 4
;	mov	LCD0VBMCN,#0xC0		; enable VBAT monitor with offset=0

	mov	LCD0CF,   #0x20		; disable VLCD/VIO supply comparator
	mov	LCD0CLKDIVH, #0		; set refresh rate
	mov	LCD0CLKDIVL, #44	
	anl	LCD0MSCN, #NOT(0x02)	; reset clear LCD bit
	orl	LCD0MSCN, #0x01		; enable LCD driver
	mov	SFRPAGE,  #0x0		; switch to SFR page 0x0
	ret

;-----------------------------------------------------------------------
Display_Number:				; display dig1, dig2, dig3, dig4
	mov	bcd, #0
	mov	bcd+1, #0
	mov	R7, #16			; convert 16-bit value

dn1:	mov	A, R4
	rlc	A
	mov	R4, A
	mov	A, R5
	rlc	A				; set C
	mov	R5, A

	mov	A, bcd
	addc	A, bcd
	da	A
	mov	bcd, A

	mov	A, bcd+1
	addc	A, bcd+1
	da 	A
	mov	bcd+1, A	
	djnz	R7, dn1
	
	mov	R1, #0x0F	
	mov	A, bcd+1
	swap	A
	anl	A, R1				; chop off higher nibble
	mov	dig1, A
	mov	A, bcd+1
	anl	A, R1
	mov	dig2, A
	
	mov	A, bcd
	swap	A
	anl	A, R1				; chop off higher nibble
	mov	dig3, A
	mov	A, bcd
	anl	A, R1
	mov	dig4, A

	mov	SFRPAGE, #0x2		; switch to SFR page 0x2
	mov	DPTR, #table7seg		; load table pointer
	
digit1:
	mov	A, dig1			; digit in A
	jnz	dig1NotBlank
	mov	A, #0x10
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D3, A;
	jmp 	digit2

dig1NotBlank:
	mov	A, dig1
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D3, A;
	jmp	dig2NotBlank

digit2:
	mov	A, dig2			; digit in A
	jnz	dig2NotBlank
	mov	A, #0x10
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D2, A;
	jmp 	digit3

dig2NotBlank:
	mov	A, dig2			; digit in A
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D2, A;
	jmp	dig3NotBlank
	
digit3:
	mov	A, dig3			; digit in A
	jnz	dig3NotBlank
	mov	A, #0x10
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D1, A;
	jmp 	digit4

dig3NotBlank:
	mov	A, dig3			; digit in A
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D1, A;

digit4:
	mov	A, dig4			; digit in A
	movc	A, @A+DPTR			; put shceme from table to A for digit
	mov	LCD0D0, A;

return:
	mov	SFRPAGE, #0x00		; switch to SFR page 0x2
	ret

;========================================================================
table7seg: ;abc.fged
	DB	11101011b			; digit 0
	DB	00001010b			; digit 1
	DB	11000111b			; digit 2
	DB	10001111b			; digit 3
	DB	00101110b			; digit 4
	DB	10101101b			; digit 5
	DB	11101101b			; digit 6
	DB	00001011b			; digit 7
	DB	11101111b			; difit 8
	DB	10101111b			; digit 9
	DB	01101111b			; digit A
	DB	11101100b			; digit b
	DB	11100001b			; digit C
	DB	11001110b			; digit d
	DB	11100101b			; digit E
	DB	01100101b			; digit F
	DB	00000000b			; blank = 10

END